@@ -43,6 +43,9 @@
 #include "interface.h"
 #include "sketchwidget.h"
 
+/* Function prototype for strcasestr(3) - a GNU function */
+char *strcasestr(const char *haystack, const char *needle);
+
 /*
  * Privates: 
  */
@@ -96,6 +99,7 @@ MainView *interface_main_view_new(AppData * data)
 	MainView *result = g_new0(MainView, 1);
 
         result->checklist_edited = FALSE;
+        result->checklist_prepend = FALSE;
         result->current_color = NULL;
         result->can_show_node_view = FALSE;
         result->node_list_longpress_path = NULL;
@@ -131,6 +135,20 @@ MainView *interface_main_view_new(AppData * data)
                          G_CALLBACK(gtk_widget_hide_on_delete),
                          NULL);
 
+	/**
+	 * We need to connect to these signals very early (before setting
+	 * up the HildonLiveSearch hook), so that we can control and
+	 * override the key presses in the main view
+	 **/
+        g_signal_connect(G_OBJECT(result->data->main_view),
+			"key-press-event",
+			G_CALLBACK(on_main_view_key_press),
+			result);
+        g_signal_connect(G_OBJECT(result->data->main_view),
+			"key-release-event",
+			G_CALLBACK(on_main_view_key_release),
+			result);
+
 	create_treeview(result);
 	create_textarea(result);
 	create_checklist(result);
@@ -149,7 +167,16 @@ MainView *interface_main_view_new(AppData * data)
 	g_signal_connect_swapped(result->treeview, "button_press_event", G_CALLBACK(cb_popup), nodemenu);
 	gtk_widget_tap_and_hold_setup(GTK_WIDGET(result->treeview), nodemenu, NULL, GTK_TAP_AND_HOLD_NONE);
 
-        gtk_container_add(GTK_CONTAINER(data->main_view), result->scrolledtree);
+	/**
+	 * Setup the main window layout:
+	 *   window -- vbox
+	 *             |- scrolledtree -- treeview
+	 *             |- livesearch
+	 **/
+	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
+        gtk_box_pack_start(GTK_BOX(vbox), result->scrolledtree, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(result->main_view_live_search), FALSE, FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(data->main_view), vbox);
 
 	result->sk = sketchwidget_new(800, 480, FALSE);
 	sketchwidget_set_undocallback(result->sk, callback_undotoggle, result);
@@ -171,8 +198,6 @@ MainView *interface_main_view_new(AppData * data)
 	result->file_name = NULL;
 
         /* Signals on the main view */
-        g_signal_connect(G_OBJECT(result->data->main_view), "key-press-event", G_CALLBACK(on_main_view_key_press), result);
-        g_signal_connect(G_OBJECT(result->data->main_view), "key-release-event", G_CALLBACK(on_main_view_key_release), result);
 	g_signal_connect(G_OBJECT(result->data->main_view), "delete_event", G_CALLBACK(close_cb), result);
 
         /* Signals on the node view */
@@ -390,8 +415,6 @@ GtkWidget *create_menu(MainView * main)
 	main->move_node_item = gtk_menu_item_new_with_label(_("Move memo"));
 	main->move_up_node_item = gtk_menu_item_new_with_label(_("Move up"));
 	main->move_down_node_item = gtk_menu_item_new_with_label(_("Move down"));
-	main->move_to_bottom_node_item = gtk_menu_item_new_with_label(_("Move to bottom"));
-	main->move_to_top_node_item = gtk_menu_item_new_with_label(_("Move to top"));
 
 	main->tools_item = gtk_menu_item_new_with_label(_("Tools"));
 
@@ -408,8 +431,6 @@ GtkWidget *create_menu(MainView * main)
 	/*move node sub-menu*/
 	gtk_menu_append(move_node_menu, main->move_up_node_item);
 	gtk_menu_append(move_node_menu, main->move_down_node_item);
-	gtk_menu_append(move_node_menu, main->move_to_bottom_node_item);
-	gtk_menu_append(move_node_menu, main->move_to_top_node_item);
 
 	gtk_menu_append(main_menu, main->tools_item);
 	gtk_menu_append(main_menu, gtk_separator_menu_item_new());
@@ -452,9 +473,7 @@ GtkWidget *create_menu(MainView * main)
 	
 	g_signal_connect(G_OBJECT(main->move_up_node_item), "activate", G_CALLBACK(callback_move_up_node), main);
 	g_signal_connect(G_OBJECT(main->move_down_node_item), "activate", G_CALLBACK(callback_move_down_node), main);
-	g_signal_connect(G_OBJECT(main->move_to_top_node_item), "activate", G_CALLBACK(callback_move_to_top_level_node), main);
-	g_signal_connect(G_OBJECT(main->move_to_bottom_node_item), "activate", G_CALLBACK(callback_move_to_bottom_level_node), main);
-	
+
 	g_signal_connect(G_OBJECT(main->tools_font), "activate", G_CALLBACK(callback_font), main);
 	g_signal_connect(G_OBJECT(main->tools_pressure), "toggled", G_CALLBACK(callback_pressure), main);
 	g_signal_connect(G_OBJECT(main->tools_color), "activate", G_CALLBACK(callback_color_invoke), main);
@@ -802,6 +821,24 @@ void create_textarea(MainView * main)
 	g_signal_connect(G_OBJECT(main->buffer), "changed", G_CALLBACK(callback_buffer_modified), main);
 }
 
+static gboolean
+livesearch_filter_func(GtkTreeModel *model, GtkTreeIter *iter,
+		gchar *text, gpointer data)
+{
+    gboolean retvalue;
+    gchar *rowtext = NULL;
+    gtk_tree_model_get(model, iter, 0, &rowtext, -1);
+
+    if (rowtext == NULL || text == NULL) {
+	    return FALSE;
+    }
+
+    retvalue = (strcasestr(rowtext, text) != NULL);
+    g_free (rowtext);
+    return retvalue;
+}
+
+
 /*
  * Create the treeview 
  */
@@ -813,11 +850,30 @@ void create_treeview(MainView * main)
 #endif*/
 	gtk_widget_show(main->scrolledtree);
 
-	GtkTreeStore *tstore = gtk_tree_store_new(N_COLUMNS, G_TYPE_STRING, GDK_TYPE_PIXBUF,
-																						G_TYPE_POINTER);
+	main->main_view_tree_store = gtk_tree_store_new(N_COLUMNS,
+			G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_POINTER);
+
+	/* Set up Hildon Live Search for the node list */
+        main->main_view_live_search = HILDON_LIVE_SEARCH(hildon_live_search_new());
+	g_signal_connect(G_OBJECT(main->main_view_live_search),
+		       "refilter",
+		       G_CALLBACK(callback_live_search_refilter),
+		       main);
+
+        main->main_view_tree_filter = GTK_TREE_MODEL_FILTER(gtk_tree_model_filter_new(
+			GTK_TREE_MODEL(main->main_view_tree_store),
+                        NULL));
+	hildon_live_search_set_filter(main->main_view_live_search,
+			main->main_view_tree_filter);
+	hildon_live_search_set_visible_func(main->main_view_live_search,
+			livesearch_filter_func, NULL, NULL);
+
+	main->treeview = hildon_gtk_tree_view_new_with_model(HILDON_UI_MODE_EDIT,
+			GTK_TREE_MODEL(main->main_view_tree_filter));
+
+	hildon_live_search_widget_hook(main->main_view_live_search,
+			GTK_WIDGET(main->data->main_view), main->treeview);
 
-	main->treeview = hildon_gtk_tree_view_new_with_model(HILDON_UI_MODE_EDIT, GTK_TREE_MODEL(tstore));
-	g_object_unref(G_OBJECT(tstore));
 
         /* Add a new button to the treeview action area */
         main->action_area_button_add = hildon_button_new_with_text(
@@ -874,7 +930,7 @@ void create_treeview(MainView * main)
 	gtk_tree_view_set_expander_column(GTK_TREE_VIEW(main->treeview), column);
 
 	gtk_tree_view_expand_all(GTK_TREE_VIEW(main->treeview));
-	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(main->treeview), TRUE);
+	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(main->treeview), FALSE);
 	gtk_tree_view_set_fixed_height_mode(GTK_TREE_VIEW(main->treeview), FALSE);
 
 	g_signal_connect(G_OBJECT(main->treeview), "button-press-event", G_CALLBACK(callback_treeview_button_press), main);
